﻿import dayjs from 'dayjs';
// type logger = {
//   level: string; // 默认为DEBUG级别
//   setLevel(newLevel:string): void;
//   shouldLog(level: string): boolean;
//   formatStack(stack: string): string;
//   log(level: string, message: string, error?: Error): void;
//   debug(message: string): void;
//   info(message: string): void;
//   warn(message: string): void;
//   error(message: string, error?: Error): void;
//   fatal(message: string, error?: Error): void;
// }

type Level = 'LOG' | 'DEBUG' | 'INFO' | 'WARN' | 'ERROR' | 'FATAL';
type LogObject = {
  level: Level;
  title: any;
  message?: any;
  error?: Error;
};

class Logger {
  private static level = 'LOG'; // 默认为LOG级别
  private static logPaletteColor = {
    default: '#78909C',
    LOG: '#78909C',
    INFO: '#5F9FD7',
    WARN: '#DC5F00',
    ERROR: '#f23232',
    FATAL: '#ff5c93',
    TIME: '#9575CD',
  };
  static logEmoji = {
    default: '📌',
    LOG: '🌈',
    INFO: '📘',
    WARN: '⚠️',
    ERROR: '⛔',
    FATAL: '🎄',
    TIME: '⏰',
  };


  private static getLogPaletteCss(level: Level) {
    return `background-color: ${this.logPaletteColor[level] || this.logPaletteColor['default']}; color: #fff; padding: 2px 4px; border-radius: 4px;}; margin-right: 10px;`;
  }

  static setLevel(newLevel: Level) {
    this.level = newLevel;
  }

  // 根据日志等级判断是否应该打印日志
  private static shouldLog(level: Level) {
    const levels = ['LOG', 'DEBUG', 'INFO', 'WARN', 'ERROR', 'FATAL'];
    return levels.indexOf(level) >= levels.indexOf(this.level);
  }

  private static formatStack(stack: string | undefined) {
    if (!stack) return '';
    // 格式化错误堆栈的逻辑
    return stack
      .split('\n')
      .map((line) => `    at ${line}`)
      .join('\n');
  }

  private static logger(infoObject: LogObject) {
    let { level, title, message, error } = infoObject;
    if (!this.shouldLog(level)) {
      return;
    }
    // const timestamp = new Date().toISOString();
    const timestamp = dayjs().format('YYYY-MM-DD HH:mm:ss SSS');
    const stack = error ? error.stack : '';

    let formattedMessage = `%c${level}%c${this.logEmoji['TIME']}${timestamp}${this.logEmoji['TIME']}%c${stack}`;
    if (typeof title === 'string') {
      title = this.logEmoji[level] + ' ' + title + ' ' + this.logEmoji[level];
    }

    // 格式化错误堆栈
    if (error&&error.stack) {
      formattedMessage += `\n${this.formatStack(error.stack)}`;
    }

    switch (level) {
      case 'LOG':
        console.log(
          formattedMessage,
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
          title??'',
          message??'',
        );
        break;
      case 'DEBUG':
        console.debug(
          formattedMessage,
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
          title??'',
          message??'',
        );
        break;
      case 'INFO':
        console.info(
          formattedMessage,
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
          title??'',
          message??'',
        );
        break;
      case 'WARN':
        console.warn(
          formattedMessage,
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
          title??'',
          message??'',
        );
        break;
      case 'ERROR':
        console.error(
          formattedMessage,
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
          title??'',
          message??'',
        );
        break;
      case 'FATAL':
        console.error(
          formattedMessage,
          title??'',
          message??'',
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
        );
        break;
      default:
        console.log(
          formattedMessage,
          this.getLogPaletteCss(level),
          this.getLogPaletteCss('FATAL'),
          '',
          title??'',
          message??'',
        );
    }
  }

  static log(...message: any[]) {
    this.logger({
      level: 'LOG',
      title: message[0],
      message: message[1]
    });
  }
  // static log(message: string | object) {
  //   this.logger('LOG', message);
  // }
  static debug(...message: any[]) {
    this.logger({
      level: 'DEBUG',
      title: message[0],
      message: message[1]
    });
  }

  static info(...message: any[]) {
    this.logger({
      level: 'INFO',
      title: message[0],
      message: message[1]
    });
  }

  static warn(...message: any[]) {
    this.logger({
      level: 'WARN',
      title: message[0],
      message: message[1]
    });
  }

  static error(error: Error,...message: any[]) {
    this.logger({
      level: 'ERROR',
      title: message[0],
      message: message[1],
      error,
    });
  }

  static fatal(error: Error,...message: any[]) {
    this.logger({
      level: 'FATAL',
      title: message[0],
      message: message[1],
      error
    });
  }
}

// 生产环境中设置日志等级
if (process.env.NODE_ENV === 'production') {
  Logger.setLevel('WARN');
}

// // 使用示例
// Logger.info('Application is starting...');
// Logger.error('Failed to load user data', new Error('Network Error'));

export default Logger;
